<?php
/**
* This is a model. It holds the logic.
* It's loaded like this in the controllers: $this->load->model("path/to/model");
* This specific model is: $this->load->model("admin/admin_model")
*
* This model is the logic behind the admin panel. It let's you manage courses, pages etc
*/
class Admin_model extends CI_Model {

	function __construct() {
		$this->load->database();
	}

	/* Get all of the courses */
	function getCourses() {
		$this->db->order_by("name", "asc");
		$query = $this->db->get('Courses');
		return $query->result();
	}
	
	/* Get all the public courses */
	function getPublicCourses() {
		$this->db->from("Courses");
		$this->db->order_by("name", "asc");
		$this->db->where("isHidden","0");
		$this->db->where("isPublic","1");
		$query = $this->db->get();
		return $query->result();
	}
	
	/* Get all the private courses for a student - If they are published and private*/
	function getPrivateCourses($username) {
		$this->db->from("Courses");
		$this->db->join("StudentCourseRegistrations", "Courses.courseID = StudentCourseRegistrations.courseID");
		$this->db->order_by("name", "asc");
		$this->db->where("userName", $username);
		$this->db->where("isPublic", "0");
		$this->db->where("isHidden","0");
		$query = $this->db->get();
		return $query->result();
	}
	
	/* Get all the courses for a student, both private from getPrivateCourses(), and public from getPublicCourses() */
	function getStudentCourses($username) {
		$private = $this->getPrivateCourses($username);
		$public = $this->getPublicCourses();
		return array_merge($private, $public);
	}

	function setCourseHidden($courseID) {
		$this->db->where("courseID", $courseID);
		$this->db->update("Courses", array("isHidden" => 1));
	}

	function unsetCourseHidden($courseID){
		$this->db->where("courseID", $courseID);
		$this->db->update("Courses", array("isHidden" => 0));
	}

	function addCourse($cid, $name, $courseData){
		$data = array(
			'courseID' => $cid ,
			'name' => $name,
			'courseData' => $courseData
			);

		$this->db->insert('Courses', $data); 
		mkdir("../courses/".$cid);

	}

	function editCourse($cid, $name, $courseData) {
		$data = array(
			'name' => $name,
			'courseData' => $courseData
		);

		$this->db->where('courseID', $cid);
		$this->db->update('Courses', $data);
	}

	function getMenu(){
		$menu = array();
		$courses = $this->db->get("Courses");
		foreach ($courses->result() as $course) {
			$menu[$course->name] = array();
			$menu[$course->name]['CID'] = $course->courseID;
			$categories = $this->db->get_where("Categories", array("courseID" => $course->courseID));

			foreach($categories->result() as $category){
				$menu[$course->name][$category->categoryName] = array();

				$subcategories = $this->db->get_where("SubCategories", array("courseID" => $course->courseID, "categoryName" => $category->categoryName));

				foreach($subcategories->result() as $subcategory){
					$menu[$course->name][$category->categoryName][$subcategory->subCategoryName] = array();

				}
			}
		}
		
		return $menu;

	}

	function getCategories($cid) {
		$this->db->order_by("orderNr", "asc");
		$query = $this->db->get_where('Categories', array("courseID" => $cid));
		return $query->result();
	}

	function addCategory($cid, $categoryName){
		$this->db->select_max("orderNr");
		$query = $this->db->get_where('Categories', array("courseID" => $cid));

		$result = $query->result();
		$data = array(
			'courseID' => $cid ,
			'categoryName' => $categoryName,
			'orderNr' => $result[0]->orderNr+1
			);

		$this->db->insert('Categories', $data); 
		mkdir("../courses/".$cid."/".$categoryName);

	}

	function getSubCategories($cid, $categoryName) {
		$query = $this->db->get_where('SubCategories', array("courseID" => $cid, "categoryName" => $categoryName));
		return $query->result();
	}

	function addSubCategory($cid, $categoryName, $subCategoryName){
		$this->db->select_max("orderNr");
		$query = $this->db->get_where('SubCategories', array("courseID" => $cid, "categoryName" => $categoryName));

		$result = $query->result();
		$data = array(
			'courseID' => $cid ,
			'subCategoryName' => $subCategoryName,
			'categoryName' => $categoryName,
			'orderNr' => $result[0]->orderNr+1
			);

		$this->db->insert('SubCategories', $data); 
		mkdir("../courses/".$cid."/".$categoryName."/".$subCategoryName);

	}

	function getExamples($cid, $categoryName, $subCategoryName) {
		$query = $this->db->get_where('Examples', array("courseID" => $cid, "categoryName" => $categoryName, "subCategoryName" => $subCategoryName));
		return $query->result();
	}

	function getQuizzes($cid, $categoryName, $subCategoryName) {
		$query = $this->db->get_where('Quizzes', array("courseID" => $cid, "categoryName" => $categoryName, "subCategoryName" => $subCategoryName));
		return $query->result();
	}

	function addExample($cid, $categoryName, $subCategoryName, $example, $description){
		$this->db->select_max("orderNr");
		$query = $this->db->get_where('Examples', array("courseID" => $cid, "categoryName" => $categoryName, "subCategoryName" => $subCategoryName));
		$result = $query->result();
		$data = array(
			'courseID' => $cid ,
			'exampleName' => $example,
			'description' => $description,
			'categoryName' => $categoryName,
			'subCategoryName' => $subCategoryName,
			'orderNr' => $result[0]->orderNr+1
			);

		$this->db->insert('Examples', $data); 

		$data = array(
			'fileName' => $cid."/".$categoryName."/".$subCategoryName."/".$example."/documentation",
			'codeLanguage' => "text",
			'fileType' => "text",
			'dataBlob' => ""
			);

		$this->db->insert('Files', $data); 

		$data = array(
			'fileName' => $cid."/".$categoryName."/".$subCategoryName."/".$example."/documentation",
			'categoryName' => $categoryName,
			'subCategoryName' => $subCategoryName,
			'exampleName' => $example,
			'courseID' => $cid,
			'columnNr' => 1,
			'orderNr' => 1
			);

		$this->db->insert('Containers', $data); 
		mkdir("../courses/".$cid."/".$categoryName."/".$subCategoryName."/".$example);

	}

	function updateExample($cid, $categoryName, $subCategoryName, $example,  $documentation, $files){

		$filearr = json_decode($files);

		foreach ($filearr as $file) {
			$output = "";
			$handle = @fopen("../courses/".$cid."/".$categoryName."/".$subCategoryName."/".$example."/".$file->filename, "r");
			if ($handle) {

				while (($buffer = fgets($handle, 4096)) !== false) {

					$buffer = str_replace("&", "&amp;", $buffer);
					$buffer = str_replace("<", "&lt;", $buffer);
					$buffer = str_replace(">", "&gt;", $buffer);

					$output .= $buffer;
				}
				if (!feof($handle)) {
					$output .= "Error: unexpected fgets() fail\n";
				}
				fclose($handle);
			}
			
			$data = array(
				'fileName' => $cid."/".$categoryName."/".$subCategoryName."/".$example."/".$file->filename ,
				'codeLanguage' => $file->lang,
				'fileType' => $file->type,
				'dataBlob' => $output
				);

			$this->db->insert('Files', $data); 

			$this->db->select_max("orderNr");
			$query = $this->db->get_where('Containers', array("courseID" => $cid, "categoryName" => $categoryName, "subCategoryName" => $subCategoryName, "exampleName" => $example, "columnNr" => $file->columnNr));
			$result = $query->result();

			$data = array(
				'fileName' => $cid."/".$categoryName."/".$subCategoryName."/".$example."/".$file->filename ,
				'categoryName' => $categoryName,
				'subCategoryName' => $subCategoryName,
				'exampleName' => $example,
				'courseID' => $cid,
				'columnNr' => $file->columnNr,
				'orderNr' => $result[0]->orderNr+1
				);

			$this->db->insert('Containers', $data); 
		}

		$data = array(
			'dataBlob' => $documentation
			);

		$this->db->where("fileName",$cid."/".$categoryName."/".$subCategoryName."/".$example."/documentation");
		$this->db->update('Files', $data); 
		

	}

	function uploadFile($files, $cid, $categoryName, $subCategoryName, $example){
		if(move_uploaded_file($files['tmp_name'], "../courses/".$cid."/".$categoryName."/".$subCategoryName."/".$example."/".$files['name'])){
			echo json_encode(array('status'=>'File was uploaded successfuly!'));
		}	
	}

}
