<?php
	Class User extends CI_Model {
		/*
		 *	Constructor
		 */
		function __construct() {
			parent::__construct();
			
			//Load required libraries and drivers
			$this->load->database();
			$this->load->library('session');
		}


		/*
		 *	This function returns the users password hint (or FALSE if user isn't logged in).
		 *  RESTRICTED-LEVEL: None
		 */ 
		public function getPasswordHint($user) {
			//Query-structure
			$this->db->select('passwdHint');
			$this->db->from('Users');
			$this->db->where('userName', $user);
			$this->db->limit(1);
			
			//Execute query
			$query = $this->db->get();
			$result = $query->result();
			
			//If a matching DB record is found.
			if($result) {
				foreach($result as $row) {
					$hint = $row->passwdHint;
					
					//Return hint
					return $hint;
				}
			}
			
			//No such user
			return FALSE;
		}


		/*
		 *	This function logs the user in (returns FALSE on fail).
		 *  RESTRICTED-LEVEL: None
		 */
		public function login($username, $password) {			
			//Generate a salted hash
			$hash = $this->getSaltedHash($password);
	
			//Query-structure
			$this->db->select('userName, name, passwd, userType, ssn, activeCourse'); // Tog bort firstLogin här.
			$this->db->from('Users');
			$this->db->where('userName', $username);
			$this->db->where('passwd', $hash);
			$this->db->limit(1);
			
			//Execute query
			$query = $this->db->get();
			$result = $query->result();
			
			//If a matching DB record is found.
			if($result) {
				//Prepare session data
				$userDetails = array();
				foreach($result as $row) {
					$userDetails = array(
						'username' => $row->userName,
						'name' => $row->name,
						'usertype' => $row->userType,
						'ssn' => $row->ssn,
						'activeCourse' => $row->activeCourse, 
					// 	'firstLogin' => $row->firstLogin
					);
				}
				
				//Set session data
				$this->session->set_userdata('authenticated', $userDetails);
				
				//Log attempt as valid
				$this->logLogin($username, 1);
				
				//Return success
				return TRUE;
			}

			//Log attempt as invalid
			$this->logLogin($username, 0);

			//Return fail
			return FALSE;
		}


		/*
		 *	This function logs the user out.
		 *  RESTRICTED-LEVEL: Self
		 */
		public function logout() {
			//Unset session data
			$this->session->unset_userdata('authenticated');
		}


		/*
		 *	This function changes the users password.
		 *  RESTRICTED-LEVEL: Self
		 */
		public function changePassword($pwdOld, $pwdNew, $pwdHint) {
			//Check that a user is logged in.
			if($this->isLoggedIn()) {
				$user = $this->getUserName();
				$oldHash = $this->getSaltedHash($pwdOld);
				$newHash = $this->getSaltedHash($pwdNew);
				
				//Validate input with database
				$this->db->select('userName');
				$this->db->from('Users');
				$this->db->where('userName', $user);
				$this->db->where('passwd', $oldHash);
				$this->db->limit(1);
				$query = $this->db->get();
				$result = $query->result();
				
				//If a matching DB record is found, update database.
				if($result) {
					$data = array(
						'passwd' => $newHash,
						'passwdHint' => $pwdHint
					);
					
					$this->db->where('userName', $user);
					$this->db->update('Users', $data);
					
					//Return Success!
					return TRUE;
				}
			}
			
			//Return error
			return FALSE;
		}


		/*
		 *	This function registers user into the database.
		 *  RESTRICTED-LEVEL: Teacher
		 */
		public function addUser($userName, $name, $ssn, $password, $userType, $pwdHint, $email) {
			//Check that a user is logged in and has the right privileges (is teacher)
			if($this->isLoggedIn() && $this->getUserType() === 'Teacher') {
				//Generate a salted hash
				$hash = $this->getSaltedHash($password);
				
				//Query-structure (All values are escaped automatically by codeigninte, producing safer queries.)
				$this->db->set('userName', $userName);
				$this->db->set('name', $name);
				$this->db->set('ssn', $ssn);
				$this->db->set('passwd', $hash);
				$this->db->set('userType', $userType);
				$this->db->set('passwdHint', $pwdHint);
				$this->db->set('email', $email);
				$result = $this->db->insert('Users');
				
				//Check for my-sql error
				if($result) {
					//Return success
					return TRUE;
				} 
			}
			
			//Return error
			return FALSE;
		}

		/* 
		 * Updates the details of a user. Takes a username and an associative array of data with the details to be changed 
		 */
		public function updateUser($username, $data) {
			$this->db->where("username", $username);
			$this->db->update("Users", $data);
		}


		/*
		 *	This function removes users from the database.
		 *  RESTRICTED-LEVEL: Teacher
		 */
		public function removeUser($userName) {
			//Check that a user is logged in, has the right privileges (is teacher) and not is the users own username.
			if($this->isLoggedIn() && $this->getUserType() === 'Teacher' && $this->getUserName() != $userName) {
				//Query-structure
				$this->db->where('userName', $userName);
				$result = $this->db->delete('Users');
				
				//Check for my-sql error
				if($result) {
					//Return success
					return TRUE;
				}
			}
			
			//Return error
			return FALSE;
		}


		/*
		 *	This reset the password for the user.
		 *  RESTRICTED-LEVEL: Teacher
		 */
		public function resetUser($userName) {
			//Check that a user is logged in, has the right privileges (is teacher) and not is the users own username.
			if($this->isLoggedIn() && $this->getUserType() === 'Teacher' && $this->getUserName() != $userName) {
				//Check user type
				$this->db->select('userName, userType, ssn, email');
				$this->db->from('Users');
				$this->db->where('userName', $username);
				$this->db->limit(1);
				$query = $this->db->get();
				$result = $query->result();
				
				//If a matching DB record is found.
				if($result) {
					//Prepare new hash depending on user-type
					$newPwdHash = '';
					
					if ($row->userType == 'Student') {
						$newPwdHash = $this->getSaltedHash($row->ssn);
					}
					else if ($row->userType == 'Teacher') {
						//$newPwdHash = $this->getSaltedHash($row->email);
						$newPwdHash = $this->getSaltedHash($row->email);
					}
					
					//Execute reset
					$data = array(
						'passwd' => $newPwdHash,
						'passwdHint' => 'default',
						'firstLogin' => 1
					);
					
					$this->db->where('userName', $userName);
					$this->db->update('Users', $data);
					
					//Return Success!
					return TRUE;
				}
			}
			
			//Return error
			return FALSE;
		}


		/*
		 *	This parses a user list from ladok and returns an array with users.
		 *  RESTRICTED-LEVEL: Teacher
		 */
		public function parseLadok($string) {
			//Check that a user is logged in and has the right privileges (is teacher).
			if($this->isLoggedIn() && $this->getUserType() === 'Teacher') {
				$userArray = array();
				
				//Populate array with users from ladok
				$ladokUsers = preg_split( '/\r\n|\r|\n/', $string);
				
				//Trim lines
				foreach ($ladokUsers as $key => $value) {
					$ladokUsers[$key] = trim($ladokUsers[$key]);
				}
				
				//Split after last name
				foreach ($ladokUsers as $key => $value) {
					$ladokUsers[$key] = explode(',', trim($ladokUsers[$key]));
				}
				
				//Replace whitespaces and tabs with divider.
				foreach ($ladokUsers as $key => $value) {
					foreach ($ladokUsers[$key] as $key2 => $value2) {
						$ladokUsers[$key][$key2] = preg_replace('/\s+/', ' ', trim($ladokUsers[$key][$key2]));
					}
				}
				
				//Generate user array
				foreach ($ladokUsers as $key => $value) {
					$temp = array(
						'ssn' => substr($ladokUsers[$key][0], 0, 11),
						'lastname' => substr($ladokUsers[$key][0], 12, strlen($ladokUsers[$key][0])),
						'firstname' => substr($ladokUsers[$key][1], 0, stripos($ladokUsers[$key][1], ' ')),
						'email' => substr($ladokUsers[$key][1], (strrpos($ladokUsers[$key][1], ' ') + 1))
					);
					$temp['username'] = substr($temp['email'], 0, (stripos($temp['email'], '@')));
					array_push($userArray, $temp);
				}
				
				//Return parsed user array
				return $userArray;
			}
			
			//If not authed
			return FALSE;
		}


		/*
		 *	Generates a salted password hash, encrypted with sha1.
		 *  RESTRICTED-LEVEL: System
		 */
		private function getSaltedHash($pwd) {
			//Salt = CodeIgniters encryption-key from config
			$salt = $this->config->item('encryption_key');
			
			//Generate SHA1 hash using salt
			$hash = sha1($salt.$pwd);
			
			return $hash;
		}


		/*
		 *	Log the login attempt.
		 *  RESTRICTED-LEVEL: System
		 */
		private function logLogin($userName, $valid) {
			$data = array(
				'userName' => $userName,
				'userAgent' => $this->session->userdata('user_agent'),
				'userIP' => $this->session->userdata('ip_address'),
				'browserID' => $this->session->userdata('session_id'),
				'success' => $valid
			);
			
			$this->db->insert('logUserLoginAttempts', $data);
		}


		/*
		 *	This function return TRUE if the user is logged in and FALSE otherwise.
		 *  RESTRICTED-LEVEL: System
		 */
		public function isLoggedIn() {
			if ($this->session->userdata('authenticated')) {
				return TRUE;
			}
			else{
				return FALSE;
			}
		}


		/*
		 *	This function returns the users type (or FALSE if user isn't logged in).
		 *  RESTRICTED-LEVEL: System
		 */
		public function getUserType() {
			if($this->isLoggedIn()) {
				$temp = $this->session->userdata('authenticated');
				return $temp['usertype'];
			}
			
			return FALSE;
		}


		/*
		 *	This function returns a boolean containing information if it is the first login.
		 *  RESTRICTED-LEVEL: System
		 */
		public function isFirstLogin() {
			if($this->isLoggedIn()) {
				$temp = $this->session->userdata('authenticated');
				if ($temp['firstLogin'] == 1) {
					return TRUE;
				}
			}
			
			return FALSE;
		}


		/*
		 *	This function returns the username (or FALSE if user isn't logged in).
		 *  RESTRICTED-LEVEL: System
		 */ 
		public function getUserName() {
			if($this->isLoggedIn()) {
				$temp = $this->session->userdata('authenticated');
				return $temp['username'];
			}
			
			return FALSE;
		}
		
		/*
		* This function fetches the active course info
		*/
		public function getActiveCourse(){
			
			if($this->isLoggedIn()) {
				$temp = $this->session->userdata('authenticated');
				$courseID = $temp['activeCourse'];
				$courseName;
			
				//Query-structure
				$this->db->select('name');
				$this->db->from('Courses');
				$this->db->where('courseID', $courseID);
				$this->db->limit(1);
				//Execute query
				$query = $this->db->get();
				$result = $query->result();

				foreach($result as $row) {
					$courseName = $row->name;
				}
				
				$data = array(
					'courseID' => $courseID,
					'courseName' => $courseName
				);
				
				return $data;
			}
			return FALSE;
		}
	}
?>